unit MainExe;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls, op_class;

type
  TMainForm = class(TForm)
    Memo1: TMemo;
    RegBtn: TButton;
    BitBtn1: TBitBtn;
    AppServer: TAppServer;
    SimpleTask: TTaskServer;
    Button1: TButton;
    Button2: TButton;
    procedure AppServerAfterRegisterTask(Sender: TObject;
      const ExpName: string);
    procedure CommandPressed(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure OnEnterCommandBtn(Sender: TObject);
    procedure RegBtnClick(Sender: TObject);
    procedure SimpleTaskExecute(Sender: TObject;
      const Args: array of TVarRec; var Retval: Pointer);
    procedure BitBtn1Click(Sender: TObject);
    procedure AppServerAvailableNotify(Sender: TObject; TaskName: string;
      IsAvailable: Boolean);
    procedure AppServerAfterUnRegisterTask(Sender: TObject;
      const TaskName: string);
    procedure FormCreate(Sender: TObject);
    procedure AppServerRegisterClient(Sender: TObject;
      const TaskName: string);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
    { Private declarations }
    FLinkList:TStringList;
    LastBtn:TBitBtn;
    procedure SortBtns;
  public
    { Public declarations }
  end;
var
  MainForm: TMainForm;

implementation

uses regdlgf;

{$R *.DFM}


procedure TMainForm.SortBtns;
var
 I:Integer;
 Y:Integer;
begin
 Y := 0;
 if FLinkList = nil Then Exit;
  for I:= 0 To FLinkList.Count -1 do
   begin
    (FLinklist.Objects[I] as TBitBtn).Top := Y;
    Inc(Y,(FLinklist.Objects[I] as TBitBtn).Height+3);
   end;
end;

procedure TMainForm.AppServerAfterRegisterTask(Sender: TObject;
  const ExpName: string);
var
 B:TBitBtn;
 I:TIcon;
begin
 if FLinkList = nil Then
   FLinkList := TStringList.Create;
 B := TBitBtn.Create(Self);
 B.Parent := Self;
  if AppServer.TaskByName[ExpName].GetTaskIcon <> 0 Then
   Begin
       I := TIcon.Create;
       try
        I.Handle := AppServer.TaskByName[ExpName].GetTaskIcon;
        B.Glyph.Width := I.Width;
        B.Glyph.Height := I.Height;
        B.Glyph.Canvas.Draw(0,0,I);
       finally
        I.Free;
       end;
   End;
 B.Height := 50;
 B.Caption := AppServer.TaskByName[ExpName].Caption;
 B.Enabled := AppServer.TaskByName[ExpName].GetTaskAvailable;
 B.Width  := Width-5;
 B.LayOut := blGlyphLeft;
 B.Tag := 1;
 B.OnClick := CommandPressed;
 B.OnEnter := OnEnterCommandBtn;
 FLinkList.AddObject(ExpName,B);
 SortBtns;
end;

procedure TMainForm.CommandPressed(Sender: TObject);
var
 I:Integer;
begin
 {Find out corresponding task name}
 I := FLinkList.IndexOfObject(Sender);
 if I = - 1 Then
  raise Exception.Create('Something wrong');
 {call corresponding task}
 AppServer.TaskByName[FLinkList.Strings[I]].Execute([nil]);
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
 CanClose := AppServer.CanCloseQuery;
end;

procedure TMainForm.OnEnterCommandBtn(Sender: TObject);
var
 I:Integer;
 S:String;
begin
 I:= FLinkList.IndexOfObject(Sender);
 if I = - 1 Then
  raise Exception.Create('Something wrong');
 {call corresponding task}
 S:= AppServer.TaskByName[FLinkList.Strings[I]].GetDescription;
 UniqueString(S);
 Memo1.Lines.Text := S;


end;

procedure TMainForm.RegBtnClick(Sender: TObject);
begin
 RegDlg.ShowModal;
end;

procedure TMainForm.SimpleTaskExecute(Sender: TObject;
  const Args: array of TVarRec; var Retval: Pointer);
begin
 messageDlg('Command Executed!',mtInformation,[mbOk],0);
end;

procedure TMainForm.BitBtn1Click(Sender: TObject);
begin
 Close;
end;

procedure TMainForm.AppServerAvailableNotify(Sender: TObject;
  TaskName: string; IsAvailable: Boolean);
var
 I:Integer;
begin
 I := FLinkList.IndexOf(TaskName);
 if I = - 1 Then
  raise Exception.Create('Something wrong');
 {call corresponding task}
 if FLinkList.Objects[I] is TBitBtn Then
    (FLinkList.Objects[I] as TBitBtn).Enabled := IsAvailable;
end;

procedure TMainForm.AppServerAfterUnRegisterTask(Sender: TObject;
  const TaskName: string);
var
 I:Integer;
begin
  I:= FLinkList.IndexOf(TaskName);
  if I <> -1 Then
    begin
    FLinkList.Objects[I].Free;
    FLinkList.Delete(I);
    end;
  SortBtns;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
 Top := 0;
 Left := 0;
end;

procedure TMainForm.AppServerRegisterClient(Sender: TObject;
  const TaskName: string);
begin
   ShowMessage(Format('Task client for task %s registered',[TaskName]));
end;

procedure TMainForm.Button1Click(Sender: TObject);
begin
 SimpleTask.RegisterTask;
 SimpleTask.NotifyAvailable(True);
end;

procedure TMainForm.Button2Click(Sender: TObject);
var
 V:Variant;
begin
 if not AppServer.HaveTask('PROPSET') Then
  raise Exception.Create('Task PROPSET not found');
  V := Color;
  AppServer.TaskByName['PROPSET'].GetProperty('COLOR',V);
  Color := V;
end;

end.
